/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.attachment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class AttachmentHandler {
    private static final String FIELD_NAME_NOT_DEFINED =
        "FIELD_NAME_NOT_DEFINED";
    private Map _attachments = null;

    public AttachmentHandler() {
        super();
        _attachments = new HashMap();
    } // public AttachmentHandler()

    public IFaceAttachmentItem getAttachment(String fieldName) {
        IFaceAttachmentItem[] attachmentsValues =
            getAttachmentValues(fieldName);
        if (attachmentsValues.length == 0)
            return null;
        return attachmentsValues[0];
    } // public IFaceAttachmentItem getAttachment(String fieldName)

    public IFaceAttachmentItem[] getAttachmentValues(String fieldName) {
        ArrayList attachmentsValues = (ArrayList) _attachments.get(fieldName);
        if (attachmentsValues == null)
            attachmentsValues = new ArrayList();
        return (IFaceAttachmentItem[]) attachmentsValues.toArray();
    } // public IFaceAttachmentItem[] getAttachmentValues(String fieldName)

    public void addAttachment(IFaceAttachmentItem attachment) {
        String fieldName = attachment.getFieldName();
        if (fieldName == null)
            fieldName = FIELD_NAME_NOT_DEFINED;
        ArrayList attachmentsValues = (ArrayList) _attachments.get(fieldName);
        if (attachmentsValues == null) {
            attachmentsValues = new ArrayList();
            _attachments.put(fieldName, attachmentsValues);
        } // if (attachmentsValues == null)
        attachmentsValues.add(attachment);
    } // public void addAttachment(IFaceAttachmentItem attachment)

    public void addAllAttachments(IFaceAttachmentItem[] attachments) {
        for (int i = 0; i < attachments.length; i++)
            addAttachment(attachments[i]);
    } // public void addAttachment(IFaceAttachmentItem attachment)

    public IteratorAttachment iterator() {
        ArrayList values = new ArrayList();
        Set keySet = _attachments.keySet();
        Iterator keyIterator = keySet.iterator();
        while (keyIterator.hasNext()) {
            String fieldName = (String) keyIterator.next();
            ArrayList attachmentsValues =
                (ArrayList) _attachments.get(fieldName);
            values.addAll(attachmentsValues);
        } // while (keyIterator.hasNext())
        return new IteratorAttachment(values.iterator());
    } // public IteratorAttachment iterator()
} // public class AttachmentHandler
