/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.base;

import java.io.Serializable;

import org.w3c.dom.Document;

import it.eng.spago.util.XMLUtil;

/**
 * Tutti gli oggetti che estendono la classe <code>AbstractXMLObject</code> sono conosciuti dal framework come oggetti aventi una rappresentazione
 * in XML . Se un oggetto estende questa classe deve implementare il metodo toElement(Document document) inserendovi
 * la logica  di generazione del XML.
 * Viene utilizzato un oggetto Document per effettuare la generazione del XML.
 *
 * @author Luigi Bellio
 */
public abstract class AbstractXMLObject implements XMLObject, Serializable {
    public AbstractXMLObject() {
        super();
    } //public AbstractXMLObject()

/**
 * Ritorna la rappresentazione XML dell'oggetto in formato stringa.
 * @return <code>String<code> la rappresentazione XML dell'oggetto.
 */
    public String toXML() {
        return XMLUtil.toXML(this, true);
    } // public String toXML()

/**
 * Ritorna la rappresentazione XML dell'oggetto in formato stringa.
 * @param inlineEntity boolean indica se la stringa XML generata deve contenere la sezione del doc type Entity.     * Ritorna la rappresentazione in XML dell'errore.Se il parametro  true allora nella stringa di ritorno
 * @return <code>String<code> la rappresentazione XML dell'oggetto.
 */
    public String toXML(boolean inlineEntity) {
        return XMLUtil.toXML(this, inlineEntity);
    } // public String toXML(boolean inlineEntity)

/**
 * Ritorna la rappresentazione XML dell'oggetto in formato stringa.
 * @param inlineEntity boolean indica se la stringa XML generata deve contenere la sezione del doc type Entity.
 * @param indent boolean indica se la stringa XML generata deve essere identata.
 * @return <code>String<code> la rappresentazione XML dell'oggetto.
 */
	public String toXML(boolean inlineEntity, boolean indent) {
		return XMLUtil.toXML(this, inlineEntity, indent);
	} // public String toXML(boolean inlineEntity)

/**
 * Ritorna la rappresentazione XML dell'oggetto in formato stringa.
 * @return <code>String<code> la rappresentazione XML dell'oggetto.
 */
    public String toXML(int level) {
        return toXML();
    } // public String toXML(int level)

/**
 * Ritorna un oggetto di tipo Document .
 * @return <code>Document<code> un oggetto di tipo Document.
 */
    public Document toDocument() {
        return XMLUtil.toDocument(this);
    } // public Document toDOM()
} // public abstract class AbstractXMLObject implements XMLObject, Serializable
