/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Questa classe ha il compito di recuperare la stringa rappresentante lo statement SQL da un
 * file XML.
 */
public class SQLStatements {
    /**
     * Questo metodo recupera lo statement SQL avente associato alla <em>key</em> name.
     * @param String nome logico dello statement SQL.
     * @return  <code>String</code> corrispondente allo statement SQL
     */
    public static String getStatement(String name) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "SQLStatements::getStatement: invocato");
        if (name == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "SQLStatements::getStatement: nome dello statement nullo");
            return null;
        } // if (name == null)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
            "SQLStatements::getStatement: name [" + name + "]");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean statementConfig = (SourceBean)configure.getFilteredSourceBeanAttribute(
            "STATEMENTS.STATEMENT", "NAME", name);
        if (statementConfig == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "SQLStatements::getStatement: statement non trovato");
            return null;
        } // if (statementConfig == null)
        String statement = (String)statementConfig.getAttribute("QUERY");
        if (statement == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "SQLStatements::getStatement: statement non trovato");
        return statement;
    } // public static String getStatement(String name)
} // public class SQLStatements
