/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.hibernate;

import java.sql.Connection;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Session;
import net.sf.hibernate.Transaction;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.tracing.TracerSingleton;

public class HibernateSession extends AbstractHibernateSession {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private DataConnection _dataConnection = null;

    /** @link dependency 
     * @stereotype instantiate*/
    /*# HibernateTransaction lnkHibernateTransaction; */

    public HibernateSession(Session nativeSession) {
        super(nativeSession);
    } // public HibernateSession(Session nativeSession)

    public Transaction beginTransaction() throws HibernateException {
        return new HibernateTransaction(super.beginTransaction());
    } // public Transaction beginTransaction() throws HibernateException

    public Connection close() throws HibernateException {
        Exception exception = null;
        try {
            super.flush();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "HibernateSession::close: ",
                ex);
            exception = ex;
        } // catch (Exception ex)
        try {
            super.close();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "HibernateSession::close: ",
                ex);
            if (exception == null)
                exception = ex;
        } // catch (Exception ex)
        try {
            if (_dataConnection != null)
                _dataConnection.close();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "HibernateSession::close: ",
                ex);
            if (exception == null)
                exception = ex;
        } // catch (Exception ex)
        _dataConnection = null;
        if (exception != null)
            throw new HibernateException(exception);
        return null;
    } // public Connection close() throws HibernateException

    public void setDataConnection(DataConnection dataConnection) {
        _dataConnection = dataConnection;
    } // public void setDataConnection(DataConnection dataConnection)
} // public class HibernateSession extends AbstractHibernateSession
