/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.sql.command.std;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.DataResultFactory;
import it.eng.spago.dbaccess.sql.result.DataResultInterface;
import it.eng.spago.dbaccess.sql.result.DefaultDataResultFactory;
import it.eng.spago.dbaccess.sql.result.InformationDataResult;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* Questa Classe rappresenta un comando SQL di DELETE per driver jdbc 2.0 o superiori
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class DeleteCommand extends SQLCommand {
    /**
    * Costruttore
    * @param <B>DataConnection</B> dataConnection - l'oggetto dataConnection legato al comando
    * @param <B>String</B> commandString - La Stringa rappresentante il comando
    */
    public DeleteCommand(DataConnection dataConnection, String commandString) {
        super(dataConnection, commandString);
    } //public DeleteCommand(DataConnection dataConnection, String commandString)

    /**
    * Questo metodo server per eseguire il comnado senza parametri
    * @return un oggetto di tipo  <B>DataResult</B> contente il risultato dell'esecuzione del comando
    * @throws <B>EMFInternalError</B> - Se si verifica qualche problema durante l'esecuzione del comando
    */
    public DataResult execute() throws EMFInternalError {
        Monitor monitor =
            MonitorFactory.start(
                "model.data-access.delete." + _commandString.toLowerCase());
        try {
            this.closeInternal();
            _stmt = getInternalConnection().createStatement();
            int rowsAffected = _stmt.executeUpdate(_commandString);
            DataResultFactory dataResultFactory =
                new DefaultDataResultFactory();
            DataResultInterface internalDataResult =
                dataResultFactory.createInformationDataResult(
                    this,
                    null,
                    InformationDataResult.CORRECT_EXECUTION,
                    rowsAffected,
                    " CORRECT EXECUTION : " + rowsAffected + " ROWS AFFECTED");
            return new DataResult(
                internalDataResult,
                DataResultInterface.INFORMATION_DATA_RESULT);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DeleteCommand::execute:",
                sqle);
            String method = "DeleteCommand :: execute() ::";
            throw Utils.generateInternalError(sqle, method);
        } // catch (SQLException sqle) try
        finally {
            monitor.stop();
        } // finally
    } //public DataResult execute() throws EMFInternalError

    /**
    * Questo metodo serve per eseguire il comando con dei parametri di input
    * @param <B>List</B> inputParameters - una lista di  <B>DataField</B> object
    * @return un oggetto di tipo  <B>DataResult</B> contente il risultato dell'esecuzione del comando
    * @throws <B>EMFInternalError</B> - Se si verifica qualche problema durante l'esecuzione del comando
    */
    public DataResult execute(List inputParameters) throws EMFInternalError {
        Monitor monitor =
            MonitorFactory.start(
                "model.data-access.delete." + _commandString.toLowerCase());
        try {
            this.closeInternal();
            _stmt = getInternalConnection().prepareStatement(_commandString);
            DataField inputParameterDataField = null;
            for (int i = 0; i < inputParameters.size(); i++) {
                inputParameterDataField = (DataField) inputParameters.get(i);
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.INFORMATION,
                    "inputParameters[" + i + "]"
                        + inputParameterDataField.getObjectValue());
                ((PreparedStatement) _stmt).setObject(
                    i + 1,
                    inputParameterDataField.getObjectValue(),
                    inputParameterDataField.getSqlType());
            } // for (int i = 0; i < inputParameters.size(); i++)
            int rowsAffected = ((PreparedStatement) _stmt).executeUpdate();
            DataResultFactory dataResultFactory =
                new DefaultDataResultFactory();
            DataResultInterface internalDataResult =
                dataResultFactory.createInformationDataResult(
                    this,
                    inputParameters,
                    InformationDataResult.CORRECT_EXECUTION,
                    rowsAffected,
                    " CORRECT EXECUTION : " + rowsAffected + " ROWS AFFECTED");
            return new DataResult(
                internalDataResult,
                DataResultInterface.INFORMATION_DATA_RESULT);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DeleteCommand::execute:",
                sqle);
            String method = "DeleteCommand :: execute(inputParameters) :: ";
            throw Utils.generateInternalError(sqle, method);
        } // catch (SQLException sqle) try
        finally {
            monitor.stop();
        } // finally
    } //public DataResult execute(List inputParameters) throws EMFInternalError
} //end Class DeleteCommand
