/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.sql.result;

import java.sql.ResultSet;
import java.util.List;

import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.legacy.LegacyScrollableDataResult;
import it.eng.spago.dbaccess.sql.result.std.CompositeDataResult;
import it.eng.spago.dbaccess.sql.result.std.DefaultScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

/**
* Questa Classe rappresenta una factory concreta per la creazione di oggetti di tipo DataResultInterface
* legati a driver jdbc di tipo 2
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class DefaultDataResultFactory implements DataResultFactory {
    /**
    * Crea un oggetto DataResultInterface di tipo ScrollableDataResult
    * @param command - Il comando che ha generato come risultato l'oggetto ScrollableDataResult che verr creato da questa funzione
    * @param inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto ScrollableDataResult che verr creato da questa funzione
    * @param rs - Il ResultSet alla base dell'oggetto ScrollableDataResult creato da questa funzione
    * @return un oggetto di tipo <B>DataResultInterface</B> - che in realt  di tipo ScrollableDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo ScrollableDataResult
    */
    public DataResultInterface createScrollableDataResult(SQLCommand command, List inputParameters, ResultSet rs, boolean std)
        throws EMFInternalError {
        if (std)
            return new DefaultScrollableDataResult(command, inputParameters, rs);
        else
            return new LegacyScrollableDataResult(command, inputParameters, rs);
    } //public DataResultInterface createScrollableDataResult(SQLCommand
    // command, List inputParameters, ResultSet rs, boolean std) throws EMFInternalError

    /**
    * Crea un oggetto DataResultInterface di tipo PunctualDataResult
    * @param command - Il comando che ha generato come risultato l'oggetto PunctualDataResult che verr creato da questa funzione
    * @param inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto PunctualDataResult che verr creato da questa funzione
    * @param punctualData - Il dataField che costituice il dato puntuale alla base dell'oggetto PunctualDataResult creato da questa funzione
    * @return un oggetto di tipo <B>DataResultInterface</B> - che in realt  di tipo PunctualDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo PunctualDataResult
    */
    public DataResultInterface createPunctualDataResult(SQLCommand command, List inputParameters, DataField punctualData)
        throws EMFInternalError {
        return new PunctualDataResult(command, inputParameters, punctualData);
    } //public DataResultInterface createPunctualDataResult(SQLCommand
    // command, List inputParameters, DataField punctualData) throws EMFInternalError

    /**
    * Crea un oggetto DataResultInterface di tipo InformationDataResult
    * @param <B>SQLCommand</B> command - Il comando che ha generato come risultato l'oggetto InformationDataResult che verr creato da questa funzione
    * @param <B>List</B> inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto InformationDataResult che verr creato da questa funzione
    * @param boolean correctExecution - true se l'esecuzione del comando  avvenuta correttamente
    * @param int rowsAffected - il numero di righe coinvolte nll'esecuzione del comando
    * @param String executionMessage - eventuale messaggio dell'InformationDataResult da creare
    * @return un oggetto di tipo <B>DataResultInterface</B> - che in realt  di tipo InformationDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo InformationDataResult
    */
    public DataResultInterface createInformationDataResult(
        SQLCommand command,
        List inputParameters,
        boolean correctExecution,
        int rowsAffected,
        String executionMessage)
        throws EMFInternalError {
        return new InformationDataResult(command, inputParameters, correctExecution, rowsAffected, executionMessage);
    } //public DataResultInterface createInformationDataResult(SQLCommand
    // command, List inputParameters, boolean
    // correctExecution, int rowsAffected, String executionMessage) throws EMFInternalError

    /**
    * Crea un oggetto DataResultInterface di tipo InformationDataResult
    * @param <B>SQLCommand</B> command - Il comando che ha generato come risultato l'oggetto CompositeDataResult che verr creato da questa funzione
    * @param <B>List</B> inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto CompositeDataResult che verr creato da questa funzione
    * @return un oggetto di tipo <B>DataResultInterface</B> - che in realt  di tipo CompositeDataResult
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo CompositeDataResult
    */
    public DataResultInterface createCompositeDataResult(SQLCommand command, List inputParameters) {
        return new CompositeDataResult(command, inputParameters);
    } //public DataResultInterface createCompositeDataResult(SQLCommand
    // command, List inputParameters)
}
