/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.action;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.security.SecurityConfigurationSingleton;
import it.eng.spago.security.SecurityException;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.coordinator.ValidationCoordinator;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 22-11-2004		  Butano	   Gestione TOKEN per il PREVENT_RESUBMIT 
* 13-12-2004          Butano               Introdotta la classe Validation per la validazione lato server
* 11-01-2005          Butano               Modifica per invocare il monitor.stop() nel caso in cui il metodo TokenHandler.isTokenValid() lanci un'eccezzione									    
**/
/**
 * @author 
 */
public class ActionCoordinator extends AbstractCoordinator {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    public ActionCoordinator(String businessType, String businessName) {
        super(businessType, businessName);
    } // public ActionCoordinator(String businessType, String businessName)

    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        Monitor monitor = MonitorFactory.start("controller.coordinator.action");
        try {
			SessionContainer sessionContainer = getRequestContainer().getSessionContainer();		
			// Verifico se  arrivato il token e se  valido
			TokenHandler.isTokenValid(sessionContainer, serviceRequest);
			//Genero e salvo il nuovo token in sessione
			Object tokenFlag = serviceRequest.getAttribute("SPAGO_GENERATETOKEN");
			if ((tokenFlag == null) ||
				!tokenFlag.toString().equalsIgnoreCase("FALSE"))
			{
				TokenHandler.saveToken(sessionContainer, serviceRequest);
			}		
	        ActionIFace action = ActionFactory.getAction(getRequestContainer());
	        synchronized (action) {
	        	// modifica A.B.
				IEngUserProfile userProfile =
	                (IEngUserProfile) getRequestContainer()
	                    .getSessionContainer()
	                    .getPermanentContainer()
	                    .getAttribute(IEngUserProfile.ENG_USER_PROFILE);
	            if (userProfile instanceof RequestContextIFace) {       
					((RequestContextIFace)userProfile).setRequestContext(this); 
	            }
				// ------      
	            boolean isActionAuthorized = true;
	            if (SecurityConfigurationSingleton
	                .getInstance()
	                .isActionChecked(getBusinessName())) {
	                if (userProfile == null) {
	                    TracerSingleton.log(
	                        Constants.NOME_MODULO,
	                        TracerSingleton.WARNING,
	                        "ActionCoordinator::service: userProfile nullo !");
	                    isActionAuthorized = false;
	                } // if (userProfile == null)
	                else {
	                    try {
	                        isActionAuthorized =
	                            userProfile.isAbleToExecuteAction(
	                                getBusinessName());
	                    } // try
	                    catch (EMFInternalError ie) {
	                        TracerSingleton.log(
	                            Constants.NOME_MODULO,
	                            TracerSingleton.CRITICAL,
	                            "ActionCoordinator::service: userProfile.isAbleToExecuteAction("
	                                + getBusinessName()
	                                + ")",
	                            (Exception) ie);
	                        isActionAuthorized = false;
	                    } // catch (EMFInternalError ie)
	                } // if (userProfile == null) else
	            } // if (SecurityConfigurationSingleton.getInstance().isActionChecked(_businessName))
	            try {
	                if (!isActionAuthorized) {
	                    TracerSingleton.log(
	                        Constants.NOME_MODULO,
	                        TracerSingleton.WARNING,
	                        "ActionCoordinator::service: autorizzazione esecuzione action ["
	                            + getBusinessName()
	                            + "] negata !");
	                    throw new SecurityException();
	                } // if (!isActionAuthorized)
	                else {
	                    TracerSingleton.log(
	                        Constants.NOME_MODULO,
	                        TracerSingleton.DEBUG,
	                        "ActionCoordinator::service: serviceRequest",
	                        serviceRequest);
	                    ((RequestContextIFace) action).setRequestContext(this);
	                    Monitor actionMonitor =
	                        MonitorFactory.start(
	                            "model.action." + getBusinessName().toLowerCase());
	                    try {
	                        if (ValidationCoordinator.validate(getBusinessType(), getBusinessName(), this))
	                        	action.service(serviceRequest, serviceResponse);
	                    } // try
	                    catch (Exception ex) {
	                        throw ex;
	                    } // catch (Exception ex)
	                    finally {
	                        actionMonitor.stop();
	                    } // finally
	                } // if (!isActionAuthorized) else
	            } // try
	            catch (Exception ex) {
	                TracerSingleton.log(
	                    Constants.NOME_MODULO,
	                    TracerSingleton.CRITICAL,
	                    "ActionCoordinator::service: eccezione nella action ["
	                        + getBusinessName()
	                        + "]");
	                throw ex;
	            } // catch (Exception ex)
	            finally {
	                ((RequestContextIFace) action).setRequestContext(null);
	                TracerSingleton.log(
	                    Constants.NOME_MODULO,
	                    TracerSingleton.DEBUG,
	                    "ActionCoordinator::service: serviceResponse",
	                    serviceResponse);
	                monitor.stop();
	            } // finally
	        } // synchronized (action)
		}
		finally {
            monitor.stop();
		}
    } // public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception
} // public class ActionCoordinator extends AbstractCoordinator
