/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.coordinator;

import java.io.Serializable;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.DefaultRequestContext;

/**
 * La classe <code>AbstractCoordinator</code> &egrave; la superclasse di tutti i coordinatori.
 * Ogni coordinator &egrave; in grado di gestire particolari richieste di servizi;due coordinators offerti dal
 * framework sono ModuleCoordinator(in grado di gestire la modalit a PAGE) e ActionCoordinator(in grado di
 * gestire la modalit ad ACTION).
 * Questa classe mette a disposizione i metodi per recuperare le instanze delle seguenti classi :
 *  <blockquote><pre>
 *  <code>RequestContainer</code> :Il contenitore di oggetti legati ai parametri della richiesta di un servizio.
 *  <code>ResponseContainer</code> :Il contenitore di oggetti legati alla riposta ad una richiesta di un servizio.
 *  <code>EMFErrorHandler</code> :Il gestore degli errori.
 *  </blockquote></pre>
 *
 * @author Luigi Bellio
 * @see it.eng.spago.base.RequestContainer
 * @see it.eng.spago.base.ResponseContainer
 * @see it.eng.spago.error.EMFErrorHandler
 */
public abstract class AbstractCoordinator extends DefaultRequestContext implements CoordinatorIFace, Serializable {
    private String _businessType = null;
    private String _businessName = null;

    public AbstractCoordinator(String businessType, String businessName) {
        _businessType = businessType;
        _businessName = businessName;
    } // public AbstractCoordinator(String businessType, String businessName)

    /**
     * Il metodo service viene invocato dopo la creazione di un'istanza del coordinator.
     * @param serviceRequest  nome del <code>SourceBean</code>
     * @param serviceResponse nome del <code>SourceBean</code>
     * @exception Exception viene lanciata se l'esecuzione del servizio viene bloccata da una anomalia
     * non gestibile.
     * @see it.eng.spago.base.SourceBean
     * @author Luigi Bellio
     */
    public abstract void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception;

    public String getBusinessType() {
        return _businessType;
    } // public String getBusinessType()

    public String getBusinessName() {
        return _businessName;
    } // public String getBusinessName()
} // public abstract class AbstractCoordinator extends DefaultRequestContext
// implements CoordinatorIFace, Serializable
