/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/

/**
 * @author 
 */
public class ConfigurablePage extends DefaultPage {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _pageName = null;
    private static final String GRAPH_ID = "id";
    private static final String GRAPH = "GRAPH";
    private static final String MODULES = "MODULES";
    private static final String DEPENDENCIES = "DEPENDENCIES";
    private static final String GRAPH_PATH = "GRAPHS.GRAPH";
    private static final String MODULE_PATH = "MODULES.MODULE";
    private static final String DEPENDENCE_PATH = "DEPENDENCIES.DEPENDENCE";
    private static final String CONDITION_PARAMETER_PATH =
        "CONDITIONS.PARAMETER";
    private static final String CONSEQUENCE_PARAMETER_PATH =
        "CONSEQUENCES.PARAMETER";
    private static final String PARAMETER_NAME = "name";
    private static final String PARAMETER_VALUE = "value";
    private static final String MODULE_ID = "id";
    private static final String MODULE_NAME = "name";
    private static final String DEPENDENCE_ID = "id";
    private static final String DEPENDENCE_SOURCE = "source";
    private static final String DEPENDENCE_TARGET = "target";

    public ConfigurablePage() {
        super();
        _pageName = null;
    } // public ConfigurablePage()

    public void init(SourceBean config) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "ConfigurablePage::instantiate: configurazione pagina prima del templating",
            config);
        _pageName = (String) config.getAttribute("name");
        SourceBean configGraph = (SourceBean) config.getAttribute(GRAPH);
        SourceBean templateGraph = instantiate(configGraph);
        try {
            config = (SourceBean) config.cloneObject();
            config.delAttribute(GRAPH);
            config.setAttribute(
                (SourceBean) templateGraph.getAttribute(MODULES));
            config.setAttribute(
                (SourceBean) templateGraph.getAttribute(DEPENDENCIES));
        } // try
        catch (SourceBeanException ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "ConfigurablePage::init: ",
                ex);
        } // catch (SourceBeanException ex)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "ConfigurablePage::instantiate: configurazione pagina dopo il templating",
            config);
        super.init(config);
    } // public void init(SourceBean config)

    protected SourceBean instantiate(SourceBean configGraph) {
        String graphId = (String) configGraph.getAttribute(GRAPH_ID);
        if (graphId == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "ConfigurablePage::instantiate: id grafo configurazione non valido !");
            return configGraph;
        } // if (graphId == null)
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean templateGraph =
            (SourceBean) configure.getFilteredSourceBeanAttribute(
                GRAPH_PATH,
                GRAPH_ID,
                graphId);
        templateGraph = (SourceBean) templateGraph.cloneObject();
        HashMap modulesMap = new HashMap();

        ArrayList templateModules =
            (ArrayList)templateGraph.getAttributeAsList(MODULE_PATH);
        for (int i = 0; i < templateModules.size(); i++) {
            SourceBean templateModule = (SourceBean) templateModules.get(i);
            String moduleId = (String) templateModule.getAttribute(MODULE_ID);
            if (moduleId == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "ConfigurablePage::instantiate: id modulo template non valido !");
                continue;
            } // if (moduleId == null)
            String moduleName =
                (String) templateModule.getAttribute(MODULE_NAME);
            modulesMap.put("{" + moduleId + "}", moduleName);
        } // for (int i = 0; i < templateModules.size(); i++)

        ArrayList configModules = (ArrayList)configGraph.getAttributeAsList(MODULE_PATH);
        for (int i = 0; i < configModules.size(); i++) {
            SourceBean configModule = (SourceBean) configModules.get(i);
            String moduleId = (String) configModule.getAttribute(MODULE_ID);
            if (moduleId == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "ConfigurablePage::instantiate: id modulo configurazione non valido !");
                continue;
            } // if (moduleId == null)
            String configModuleName =
                (String) configModule.getAttribute(MODULE_NAME);
            if ((configModuleName != null) && !(configModuleName.equals("")))
                modulesMap.put("{" + moduleId + "}", configModuleName);
            SourceBean templateModule =
                (SourceBean) templateGraph.getFilteredSourceBeanAttribute(
                    MODULE_PATH,
                    MODULE_ID,
                    moduleId);
            try {
                templateModule.updContainedAttributes(
                    configModule.getContainedAttributes());
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "ConfigurablePage::instantiate: errore aggiornamento modulo template ["
                        + moduleId
                        + "]",
                    ex);
            } // catch (SourceBeanException ex)
        } // for (int i = 0; i < configModules.size(); i++)

        List configDependencies =
            configGraph.getAttributeAsList(DEPENDENCE_PATH);
        Iterator iteratorConfig = configDependencies.iterator();
        while (iteratorConfig.hasNext()) {
            SourceBean configDependence =
                (SourceBean) iteratorConfig.next();
            String dependenceId =
                (String) configDependence.getAttribute(DEPENDENCE_ID);
            if (dependenceId == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "ConfigurablePage::instantiate: id dipendenza non valido !");
                continue;
            } // if (dependenceId == null)
            SourceBean templateDependence =
                (SourceBean) templateGraph.getFilteredSourceBeanAttribute(
                    DEPENDENCE_PATH,
                    DEPENDENCE_ID,
                    dependenceId);
            try {
                templateDependence.updContainedAttributes(
                    configDependence.getContainedAttributes());
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "ConfigurablePage::instantiate: errore aggiornamento template dipendenza ["
                        + dependenceId
                        + "]",
                    ex);
            } // catch (SourceBeanException ex)
        } // for (int i = 0; i < configDependencies(); i++)

        List templateDependencies =
            templateGraph.getAttributeAsList(DEPENDENCE_PATH);
        Iterator iteratorTemplate = templateDependencies.iterator();
        while (iteratorTemplate.hasNext()) {       
            SourceBean templateDependence =
                (SourceBean) iteratorTemplate.next();
            String templateDependenceSource =
                (String) templateDependence.getAttribute(DEPENDENCE_SOURCE);
            try {
                if ((templateDependenceSource == null)
                    || (templateDependenceSource.equals("")))
                    templateDependence.updAttribute(
                        DEPENDENCE_SOURCE,
                        _pageName);
                else
                    templateDependence.updAttribute(
                        DEPENDENCE_SOURCE,
                        resolvePath(templateDependenceSource, modulesMap));
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "ConfigurablePage::instantiate: errore aggiornamento modulo source template ["
                        + templateDependenceSource
                        + "]",
                    ex);
            } // catch (SourceBeanException ex)
            String templateDependenceTarget =
                (String) templateDependence.getAttribute(DEPENDENCE_TARGET);
            try {
                templateDependence.updAttribute(
                    DEPENDENCE_TARGET,
                    resolvePath(templateDependenceTarget, modulesMap));
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "ConfigurablePage::instantiate: errore aggiornamento modulo target template ["
                        + templateDependenceTarget
                        + "]",
                    ex);
            } // catch (SourceBeanException ex)
            List conditionParameters =
                templateDependence.getAttributeAsList(
                    CONDITION_PARAMETER_PATH);
            Iterator iteratorCondition = conditionParameters.iterator();
            while (iteratorCondition.hasNext()) {       
                SourceBean conditionParameter =
                    (SourceBean) iteratorCondition.next();
                String conditionParameterName =
                    (String) conditionParameter.getAttribute(PARAMETER_NAME);
                try {
                    conditionParameter.updAttribute(
                        PARAMETER_NAME,
                        resolvePath(conditionParameterName, modulesMap));
                } // try
                catch (SourceBeanException ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "ConfigurablePage::instantiate: errore aggiornamento nome parametro condizione ["
                            + conditionParameterName
                            + "]",
                        ex);
                } // catch (SourceBeanException ex)
                String conditionParameterValue =
                    (String) conditionParameter.getAttribute(PARAMETER_VALUE);
                try {
                    conditionParameter.updAttribute(
                        PARAMETER_VALUE,
                        resolvePath(conditionParameterValue, modulesMap));
                } // try
                catch (SourceBeanException ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "ConfigurablePage::instantiate: errore aggiornamento valore parametro condizione ["
                            + conditionParameterValue
                            + "]",
                        ex);
                } // catch (SourceBeanException ex)
            } // for (int j = 0; j < conditionParameters.size(); j++)
            List consequenceParameters =
                templateDependence.getAttributeAsList(
                    CONSEQUENCE_PARAMETER_PATH);
            Iterator iteratorConsequence = consequenceParameters.iterator();
            while (iteratorConsequence.hasNext()){
                SourceBean consequenceParameter =
                    (SourceBean) iteratorConsequence.next();
                String consequenceParameterName =
                    (String) consequenceParameter.getAttribute(PARAMETER_NAME);
                try {
                    consequenceParameter.updAttribute(
                        PARAMETER_NAME,
                        resolvePath(consequenceParameterName, modulesMap));
                } // try
                catch (SourceBeanException ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "ConfigurablePage::instantiate: errore aggiornamento nome parametro conseguenza ["
                            + consequenceParameterName
                            + "]",
                        ex);
                } // catch (SourceBeanException ex)
                String consequenceParameterValue =
                    (String) consequenceParameter.getAttribute(PARAMETER_VALUE);
                try {
                    consequenceParameter.updAttribute(
                        PARAMETER_VALUE,
                        resolvePath(consequenceParameterValue, modulesMap));
                } // try
                catch (SourceBeanException ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "ConfigurablePage::instantiate: errore aggiornamento valore parametro conseguenza ["
                            + consequenceParameterValue
                            + "]",
                        ex);
                } // catch (SourceBeanException ex)
            } // for (int j = 0; j < consequenceParameters.size(); j++)
        } // for (int i = 0; i < configDependencies(); i++)

        return templateGraph;
    } // protected SourceBean override(SourceBean config)

    private String resolvePath(String toResolve, Map parameters) {
        if ((toResolve == null) || (parameters == null))
            return toResolve;
        StringBuffer resolved = new StringBuffer();
        int startIndex = toResolve.indexOf("{");
        while (startIndex != -1) {
            resolved.append(toResolve.substring(0, startIndex));
            int endIndex = toResolve.indexOf("}", startIndex);
            String moduleId = toResolve.substring(startIndex, endIndex + 1);
            String moduleName = (String) parameters.get(moduleId);
            if (moduleName == null)
                resolved.append(toResolve.substring(startIndex, endIndex + 1));
            else
                resolved.append(moduleName);
            toResolve = toResolve.substring(endIndex + 1, toResolve.length());
            startIndex = toResolve.indexOf("{");
        } // while (parameterIndex != -1)
        resolved.append(toResolve);
        return resolved.toString();
    } // private String resolvePath(String toResolve, Map parameters)
} // public class ConfigurablePage extends DefaultPage
