/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/

/**
 * @author 
 */
public class ModuleDependencies {
    private ModuleDependencies() {
        super();
    } // private ModuleDependencies()

    public static Collection getTargetRequests(
        RequestContainer requestContainer,
        ResponseContainer responseContainer,
        String sourceName,
        SourceBean pageConfig) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "ModuleDependencies::getTargetRequests: sourceName ["
                + sourceName
                + "]");
        ArrayList targetRequests = new ArrayList();
        String page =
            (String) requestContainer.getServiceRequest().getAttribute("PAGE");
        if (pageConfig == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "ModuleDependencies::getTargetRequests: pageConfig ["
                    + page
                    + "] nullo");
            return targetRequests;
        } // if (pageConfig == null)
        if (sourceName != null) {
            SourceBean moduleConfig =
                (SourceBean) pageConfig.getFilteredSourceBeanAttribute(
                    "MODULES.MODULE",
                    "NAME",
                    sourceName);
            if (moduleConfig == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: module ["
                        + sourceName
                        + "] non censito nella pagina");
                return targetRequests;
            } // if (moduleConfig == null)
        } // if (module != null)
        List dependencies = null;
        if (sourceName == null)
            dependencies =
                pageConfig.getFilteredSourceBeanAttributeAsList(
                    "DEPENDENCIES.DEPENDENCE",
                    "SOURCE",
                    page);
        else
            dependencies =
                pageConfig.getFilteredSourceBeanAttributeAsList(
                    "DEPENDENCIES.DEPENDENCE",
                    "SOURCE",
                    sourceName);
        if (dependencies.size() == 0)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "ModuleDependencies::getTargetRequests: nessuna dipendenza trovata");
        for (int i = 0; i < dependencies.size(); i++) {
            SourceBean dependence = (SourceBean) dependencies.get(i);
            List conditions =
                dependence.getAttributeAsList("CONDITIONS.PARAMETER");
            if (conditions.size() == 0)
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "ModuleDependencies::getTargetRequests: nessuna condizione trovata");
            boolean conditionVerified = true;
            for (int j = 0; j < conditions.size(); j++) {
                SourceBean condition = (SourceBean) conditions.get(j);
                String parameterName = (String) condition.getAttribute("NAME");
                String parameterScope =
                    (String) condition.getAttribute("SCOPE");
                String parameterValue =
                    (String) condition.getAttribute("VALUE");
                String inParameterValue = null;
                Object parameterValueObject =
                    ContextScooping.getScopedParameter(
                        requestContainer,
                        responseContainer,
                        parameterName,
                        parameterScope);
                if (parameterValueObject != null)
                    inParameterValue = parameterValueObject.toString();
                if (parameterValue.equalsIgnoreCase("AF_DEFINED")) {
                    if (inParameterValue == null) {
                        conditionVerified = false;
                        break;
                    } // if (inParameterValue == null)
                    continue;
                } // if (parameterValue.equalsIgnoreCase("AF_DEFINED"))
                if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED")) {
                    if (inParameterValue != null) {
                        conditionVerified = false;
                        break;
                    } // if (inParameterValue != null)
                    continue;
                } // if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED"))
                if (!(parameterValue.equalsIgnoreCase(inParameterValue))) {
                    conditionVerified = false;
                    break;
                } // if (!(parameterValue.equalsIgnoreCase(inParameterValue)))
            } // for (int j = 0; conditions.size(); j++)
            if (conditionVerified) {
                List consequences =
                    dependence.getAttributeAsList("CONSEQUENCES.PARAMETER");
                if (consequences.size() == 0)
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.WARNING,
                        "ModuleDependencies::getTargetRequests: nessuna conseguenza trovata");
                SourceBean targetRequest =
                    (SourceBean) requestContainer
                        .getServiceRequest()
                        .cloneObject();
                String targetName = (String) dependence.getAttribute("TARGET");
                try {
                    targetRequest.updAttribute("AF_MODULE_NAME", targetName);
                } // try
                catch (SourceBeanException sbe) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "ModuleDependencies::getTargetRequests: targetRequest.updAttribute(\"AF_MODULE_NAME\", "
                            + targetName
                            + ")",
                        sbe);
                } // catch (SourceBeanException sbe)
                for (int j = 0; j < consequences.size(); j++) {
                    SourceBean consequence = (SourceBean) consequences.get(j);
                    String parameterName =
                        (String) consequence.getAttribute("NAME");
                    String parameterScope =
                        (String) consequence.getAttribute("SCOPE");
                    String parameterType =
                        (String) consequence.getAttribute("TYPE");
                    String parameterValue =
                        (String) consequence.getAttribute("VALUE");
                    Object inParameterValue = null;
                    if (parameterType.equalsIgnoreCase("ABSOLUTE"))
                        inParameterValue = parameterValue;
                    else
                        inParameterValue =
                            ContextScooping.getScopedParameter(
                                requestContainer,
                                responseContainer,
                                parameterValue,
                                parameterScope);
                    String parameterAppend =
                        (String) consequence.getAttribute("APPEND");
                    boolean append =
                        ((parameterAppend != null)
                            && parameterAppend.equalsIgnoreCase("true"));
                    try {
                        if (inParameterValue == null)
                            targetRequest.delAttribute(parameterName);
                        else if (inParameterValue instanceof SourceBean)
                            if (append)
                                targetRequest.setAttribute(
                                    (SourceBean) inParameterValue);
                            else
                                targetRequest.updAttribute(
                                    (SourceBean) inParameterValue);
                        else if (append)
                            targetRequest.setAttribute(
                                parameterName,
                                inParameterValue);
                        else
                            targetRequest.updAttribute(
                                parameterName,
                                inParameterValue);
                    } // try
                    catch (SourceBeanException sbe) {
                        TracerSingleton.log(
                            Constants.NOME_MODULO,
                            TracerSingleton.CRITICAL,
                            "ModuleDependencies::getTargetRequests: targetRequest.updAttribute("
                                + parameterName
                                + ", "
                                + inParameterValue
                                + ")",
                            sbe);
                    } // catch (SourceBeanException sbe)
                } // for (int j = 0; j < consequences.size(); j++)
                targetRequests.add(targetRequest);
            } // if (conditionVerified)
        } // for (int i = 0; i < dependencies.size(); i++)
        return targetRequests;
    } // public static Collection getTargetRequests(RequestContainer requestContainer, ResponseContainer responseContainer, String sourceName, SourceBean pageConfig)
} // public class ModuleDependencies
