/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.condition;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class EventConditionDriverManager implements InitializerIFace {
    private SourceBean _config = null;
    private static HashMap _eventConditionDrivers = new HashMap();

    private EventConditionDriverManager() {
        super();
        _config = null;
    } // private EventConditionDriverManager()

    public void init(SourceBean config) {
        _config = config;
    } // public void init(SourceBean config)

    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()

    public static void registerEventConditionDriver(EventConditionDriverIFace eventConditionDriver) {
        if (eventConditionDriver == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventConditionDriverManager::registerEventConditionDriver: eventConditionDriver nullo");
            return;
        } // if (eventConditionDriver == null)
        String eventConditionKey = eventConditionDriver.getEventCondition().getKey();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventConditionDriverManager::registerEventConditionDriver: eventConditionKey [" + eventConditionKey + "]");
        synchronized(EventConditionDriverManager.class) {
            if (_eventConditionDrivers.containsKey(eventConditionKey)) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventConditionDriverManager::registerEventConditionDriver: eventConditionDriver gi registrato");
                return;
            } // if (_eventConditionDrivers.containsKey(eventConditionKey))
            _eventConditionDrivers.put(eventConditionKey, eventConditionDriver);
        } // synchronized(EventConditionDriverManager.class)
    } // public static void registerEventConditionDriver(EventConditionDriverIFace

    // eventConditionDriver)
    public static Collection getEventConditions(RequestContainer request, ResponseContainer response) {
        Collection eventConditions = new ArrayList();
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventConditionDriverManager::getEventConditions: request nullo");
            return eventConditions;
        } // if (request == null)
        synchronized(EventConditionDriverManager.class) {
            Collection eventConditionDrivers = _eventConditionDrivers.values();
            Iterator eventConditionDriversIterator = eventConditionDrivers.iterator();
            while (eventConditionDriversIterator.hasNext()) {
                EventConditionDriverIFace eventConditionDriver =
                    (EventConditionDriverIFace)eventConditionDriversIterator.next();
                if (eventConditionDriver.acceptsService(request, response)) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                        "EventConditionDriverManager::getEventConditions: eventConditionDriver trovato [" +
                        eventConditionDriver.getClass().getName() + "]");
                    eventConditions.add(eventConditionDriver.getEventCondition());
                } // if (eventConditionDriver.acceptsURL(request, response))
            } // while (eventConditionDriversIterator.hasNext())
        } // synchronized(EventConditionDriverManager.class)
        if (eventConditions.size() == 0)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "EventConditionDriverManager::getEventConditions: nessuna condizione trovata");
        return eventConditions;
    } // public static Collection getEventCondition(RequestContainer request,

    // ResponseContainer response)
    public static void deregisterEventConditionDriver(EventConditionDriverIFace eventConditionDriver) {
        if (eventConditionDriver == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventConditionDriverManager::deregisterEventConditionDriver: eventConditionDriver nullo");
            return;
        } // if (eventConditionDriver == null)
        String eventConditionKey = eventConditionDriver.getEventCondition().getKey();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventConditionDriverManager::deregisterEventConditionDriver: eventConditionKey [" + eventConditionKey + "]");
        synchronized(EventConditionDriverManager.class) {
            if (!(_eventConditionDrivers.containsKey(eventConditionKey))) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventConditionDriverManager::deregisterEventConditionDriver: eventConditionDriver gi deregistrato");
                return;
            } // if (!(_eventConditionDrivers.containsKey(EventConditionDriverName)))
            _eventConditionDrivers.remove(eventConditionKey);
        } // synchronized(EventConditionDriverManager.class)
    } // public static void deregisterEventConditionDriver(EventConditionDriverIFace
    // eventConditionDriver)
} // public class EventConditionDriverManager implements InitializerIFace
