/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.condition.impl;

import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.event.condition.AbstractEventConditionDriver;
import it.eng.spago.event.condition.EventConditionIFace;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

public class DefaultEventConditionDriver extends AbstractEventConditionDriver {
    private EventConditionIFace _eventCondition = null;

    public DefaultEventConditionDriver() {
        super();
        _eventCondition = null;
    } // public DefaultEventConditionDriver()

    public void init(SourceBean config) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "DefaultEventConditionDriver::init: invocato");
        super.init(config);
        try {
            String eventConditionClassName = (String)config.getAttribute("CLASS");
            _eventCondition = (EventConditionIFace)Class.forName(eventConditionClassName).newInstance();
            ((InitializerIFace)_eventCondition).init(config);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DefaultEventConditionDriver::init:", ex);
        } // catch (Exception ex) try
    } // public void init(SourceBean config)

    public boolean acceptsService(RequestContainer request, ResponseContainer response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "DefaultEventConditionDriver::acceptsService: invocato");
        List parameters = getConfig().getAttributeAsList("PARAMETERS.PARAMETER");
        if (parameters == null)
            return true;
        for (int i = 0; i < parameters.size(); i++) {
            SourceBean parameter = (SourceBean)parameters.get(i);
            String parameterName = (String)parameter.getAttribute("NAME");
            String parameterScope = (String)parameter.getAttribute("SCOPE");
            String parameterValue = (String)parameter.getAttribute("VALUE");
            String inParameterValue = null;
            if (parameterScope.equalsIgnoreCase("CONFIG"))
                inParameterValue = (String)getConfig().getAttribute(parameterName);
            else {
                Object parameterValueObject = ContextScooping.getScopedParameter(request, response,
                    parameterName, parameterScope);
                if (parameterValueObject != null)
                    inParameterValue = parameterValueObject.toString();
            } // else
            if (parameterValue.equalsIgnoreCase("AF_DEFINED")) {
                if (inParameterValue == null)
                    return false;
                continue;
            } // if (parameterValue.equalsIgnoreCase("AF_DEFINED"))
            if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED")) {
                if (inParameterValue != null)
                    return false;
                continue;
            } // if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED"))
            if (!(parameterValue.equalsIgnoreCase(inParameterValue)))
                return false;
        } // for (int i = 0; i < parameters.size(); i++)
        return true;
    } // public boolean acceptsService(RequestContainer request, ResponseContainer response)

    public EventConditionIFace getEventCondition() {
        return _eventCondition;
    } // public EventConditionIFace getEventCondition()
} // public class DefaultEventConditionDriver extends AbstractEventConditionDriver
