/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.handler.impl;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.InformationDataResult;
import it.eng.spago.event.condition.EventConditionIFace;
import it.eng.spago.event.handler.AbstractEventHandler;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

public class DefaultNotifierEventHandler extends AbstractEventHandler {
    public static final String PARAMETER_SEPARATOR = "#";
    public static final String EVENT_KEY_SEPARATOR = "|";

    public DefaultNotifierEventHandler() {
        super();
    } // public DefaultNotifierEventPlugin()

    public void init(SourceBean config) {
        super.init(config);
    } // public void init(SourceBean config)

    public void service(Collection eventConditions, RequestContainer request, ResponseContainer response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "DefaultNotifierEventPlugin::service: servizio non implementato");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "DefaultNotifierEventPlugin::init: service", getConfig());
        if (eventConditions == null)
            return;
        ArrayList triggeredEventConditionKeys = new ArrayList();
        if (eventConditions != null) {
            Iterator eventConditionIterator = eventConditions.iterator();
            while (eventConditionIterator.hasNext()) {
                EventConditionIFace eventCondition = (EventConditionIFace)eventConditionIterator.next();
                triggeredEventConditionKeys.add(eventCondition.getKey());
            } // while (eventConditionIterator.hasNext())
        } // if (eventConditions != null)
        DataConnectionManager dataConnectionManager = null;
        DataConnection dataConnection = null;
        SQLCommand insertCommand = null;
        try {
            dataConnectionManager = DataConnectionManager.getInstance();
            dataConnection = dataConnectionManager.getConnection((String)getConfig().getAttribute("POOL"));
            String insertStatement = (String)getConfig().getAttribute("SQL");
            insertCommand = dataConnection.createInsertCommand(SQLStatements.getStatement(insertStatement));
            List inputParameters = new ArrayList();
            List fields = getConfig().getAttributeAsList("FIELDS.PARAMETERS");
            for (int i = 0; i < fields.size(); i++) {
                SourceBean field = (SourceBean)fields.get(i);
                StringBuffer parametersValue = new StringBuffer();
                List parameters = field.getAttributeAsList("PARAMETER");
                String parameterSeparator = "";
                for (int j = 0; j < parameters.size(); j++) {
                    parametersValue.append(parameterSeparator);
                    parameterSeparator = PARAMETER_SEPARATOR;
                    SourceBean parameter = (SourceBean)parameters.get(j);
                    String parameterName = (String)parameter.getAttribute("NAME");
                    String parameterScope = (String)parameter.getAttribute("SCOPE");
                    String parameterValue = null;
                    if (parameterScope.equalsIgnoreCase("LOCAL")) {
                        if (parameterName.equalsIgnoreCase("EVENT")) {
                            StringBuffer parameterValueBuffer = new StringBuffer();
                            String eventKeySeparator = "";
                            for (int k = 0; k < triggeredEventConditionKeys.size(); k++) {
                                parameterValueBuffer.append(eventKeySeparator);
                                eventKeySeparator = EVENT_KEY_SEPARATOR;
                                parameterValueBuffer.append(triggeredEventConditionKeys.get(k));
                            } // for (int k = 0; k <
                            // triggeredEventConditionKeys.size(); k++)
                            parameterValue = parameterValueBuffer.toString();
                        } // if (parameterName.equalsIgnoreCase("EVENT"))
                    } // if (parameterScope.equalsIgnoreCase("LOCAL"))
                    else if (parameterScope.equalsIgnoreCase("CONFIG"))
                        parameterValue = (String)getConfig().getAttribute(parameterName);
                    else if (parameterScope.equalsIgnoreCase("ADAPTER_RESPONSE"))
                        parameterValue = (String)response.getAttribute(parameterName);
                    else if (parameterScope.equalsIgnoreCase("SERVICE_RESPONSE"))
                        parameterValue = (String)request.getServiceRequest().getAttribute(parameterName);
                    else {
                        Object parameterValueObject = ContextScooping.getScopedParameter(request, response,
                            parameterName, parameterScope);
                        if (parameterValueObject != null)
                            parameterValue = parameterValueObject.toString();
                    } // else
                    if (parameterValue != null)
                        parametersValue.append(parameterValue);
                } // for (int j = 0; j < parameters.size(); j++)
                inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, parametersValue.toString()));
            } // for (int i = 0; i < fields.size(); i++)
            DataResult dataResult = insertCommand.execute(inputParameters);
            InformationDataResult informationDataResult = (InformationDataResult)dataResult.getDataObject();
            if (informationDataResult.getAffectedRows() == 0)
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
                    "DefaultNotifierEventPlugin::service: errore inserimento evento");
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DefaultNotifierEventPlugin::service:", ex);
        } // catch (Exception ex) try
        finally {
            if (insertCommand != null)
                try {
                    insertCommand.close();
                } // try
                catch (Exception ex) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "DefaultNotifierEventPlugin::service: insertCommand.close()", ex);
                } // catch (Exception ex)
            if (dataConnection != null)
                try {
                    dataConnection.close();
                } // try
                catch (Exception ex) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "DefaultNotifierEventPlugin::service: dataConnection.close()", ex);
                } // catch (Exception ex)
        } // finally try
    } // public void service(Collection eventConditions, RequestContainer
    // request, ResponseContainer response)
} // public class DefaultNotifierEventPlugin extends AbstractEventPlugin
