package it.eng.spago.message;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.tracing.TracerSingleton;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * Questa classe permette il recupero di stringhe censite in  alcuni files di properties.
 */
public class MessageBundle {
	
	private static final String DEFAULT_BUNDLE = "messages";
	
    private MessageBundle() {
        super();
    } // private MessageBundle()

	/**
	 * Questo metodo recupera una stringa individuata da un codice e da alcune propriet dell'utente
	 * che ha richiesto il servizio (language,country).
	 * <p>
	 * @param code il codice associato alla stringa.
	 * @return la stringa recuperata da un file di properties.
	 */
	public static String getMessage(String code) {
		return getMessage(code, DEFAULT_BUNDLE);
	}

    /**
     * Questo metodo recupera una stringa individuata da un codice e da alcune propriet dell'utente
     * che ha richiesto il servizio (language,country).
     * <p>
     * @param code il codice associato alla stringa.
     * @param bundle il bundle da cui recuperare la stringa.
     * @return la stringa recuperata da un file di properties.
     */
    public static String getMessage(String code, String bundle) {
        String language = null;
        String country = null;
        RequestContainer requestContainer = RequestContainer.getRequestContainer();
        if (requestContainer == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "MessageBundle::getMessage: requestContainer nullo");
        else {
            SessionContainer sessionContainer = requestContainer.getSessionContainer().getPermanentContainer();
            language = (String)sessionContainer.getAttribute("AF_LANGUAGE");
            country = (String)sessionContainer.getAttribute("AF_COUNTRY");
        } // if (requestContainer != null)
        if (language == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "MessageBundle::getMessage: language non specificato");
            language = "en";
        } // if (language == null)
        if (country == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "MessageBundle::getMessage: country non specificato");
            country = "US";
        } // if (country == null)
        Locale currentLocale = new Locale(language, country);
        ResourceBundle messages = ResourceBundle.getBundle(bundle, currentLocale);
        if (messages == null) {
            return null;
        } // if (messages == null)
        String message = code;
        try {
            message = messages.getString(code);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "MessageBundle::getMessage: codice [" + code + "] non trovato");
        } // catch (Exception ex)
        return message;
    } // public String getMessage(String code)
} // public class MessageBundle
