/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.security;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author zoppello
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class GenericLoginHandler {
    /**
     * @see it.eng.spago.dispatching.service.ServiceIFace#service(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
     */
    public void service(RequestContainer requestContainer, SourceBean request, SourceBean response) throws EMFInternalError {
        IAuthenticationHandler authenticationHandler = getAuthenticationHadler();
        IAuthorizationHandler authorizationHandler = getAuthorizationHandler();
        Object userPrincipal = authenticationHandler.authenticate(request, response);
        authorizationHandler.putAuhtorizationProfile(userPrincipal, requestContainer);
    } // public void service(RequestContainer requestContainer, SourceBean request, SourceBean response) throws EMFInternalError

    private IAuthenticationHandler getAuthenticationHadler() throws EMFInternalError {
        SourceBean authenticationHandler = (SourceBean)
            ConfigSingleton.getInstance().getAttribute("SECURITY.AUTHENTICATION_HANDLER");
        String authenticationHandlerClassName = (String)authenticationHandler.getCharacters();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericLoginHandler::getAuthenticationHadler: authenticationHandlerClassName [ " +
            authenticationHandlerClassName + " ] ");
        try {
            return (IAuthenticationHandler)Class.forName(authenticationHandlerClassName).newInstance();
        } // try
        catch (InstantiationException ie) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, ie);
        } // catch (InstantiationException ie)
        catch (IllegalAccessException iae) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, iae);
        } // catch (InstantiationException iae)
        catch (ClassNotFoundException cnfe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, cnfe);
        } // catch (InstantiationException cnfe)
    } // private IAuthenticationHandler getAuthenticationHadler() throws EMFInternalError

    private IAuthorizationHandler getAuthorizationHandler() throws EMFInternalError {
        SourceBean authorizationHandler = (SourceBean)
            ConfigSingleton.getInstance().getAttribute("SECURITY.AUTHORIZATION_HANDLER");
        String authorizationHandlerClassName = (String)authorizationHandler.getCharacters();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "GenericLoginHandler::getAuthenticationHadler: authorizationHandlerClassName [ " +
            authorizationHandlerClassName + " ] ");
        try {
            return (IAuthorizationHandler)Class.forName(authorizationHandlerClassName).newInstance();
        } // try
        catch (InstantiationException ie) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, ie);
        } // catch (InstantiationException ie)
        catch (IllegalAccessException iae) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, iae);
        } // catch (InstantiationException iae)
        catch (ClassNotFoundException cnfe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, cnfe);
        } // catch (InstantiationException cnfe)
    } // private IAuthorizationHandler getAuthorizationHandler() throws EMFInternalError
} // public void service(RequestContainer requestContainer, SourceBean request, SourceBean response) throws EMFInternalError
