/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.security;

import java.io.Serializable;
import java.util.Collection;

import it.eng.spago.error.EMFInternalError;

/**
 * @author Andrea Zoppello
 *
 * EngUserProfile - Definisce l'interfaccia per gli oggetti che rappresentano il profilo dell'utente
 *
 */
public interface IEngUserProfile extends Serializable {
    /**
     * Method getUserUniqueIdentifier.
     * @return Object
     */
    public Object getUserUniqueIdentifier();

    /**
     * Method getUserAttribute.
     * @param attributeName
     * @return Object
     *
     * Ritorna il valore dell'attributo dell'utente
     */
    public Object getUserAttribute(String attributeName) throws EMFInternalError;

    /**
     * Method hasRole
     * @param roleName
     * @return boolean
     *
     * Ritorna true se l'utente  associato a quel ruolo
     */
    public boolean hasRole(String roleName) throws EMFInternalError;

    /**
     * Method getRoles.
     * @return List
     *
     * Ritorna tutti i ruoli con la quale l'utente ha una relazione di behaviour
     */
    public Collection getRoles() throws EMFInternalError;

    /**
     * Method getFunctionalities.
     * @return Collection
     */
    public Collection getFunctionalities() throws EMFInternalError;

    /**
     * Method isAbleToExecuteAction.
     * @param actionName
     * @return boolean
     * @throws EMFInternalError
     *
     * Ritorna True se l'utente  abilitato ad eseguire la action actionName
     */
    public boolean isAbleToExecuteAction(String actionName) throws EMFInternalError;

    /**
     * Method isAbleToExecuteModuleInPage.
     * @param pageName
     * @param moduleName
     * @return boolean
     * @throws EMFInternalError
     *
     * Ritorna True se l'utente  abilitato ad eseguire il modulo moduleName
     * nella pagina pageName
     */
    public boolean isAbleToExecuteModuleInPage(String pageName, String moduleName) throws EMFInternalError;

    /**
     * Method setApplication.
     * @param applicationName
     * @throws EMFInternalError
     */
    public void setApplication(String applicationName) throws EMFInternalError;

    public final static String ENG_USER_PROFILE = "ENG_USER_PROFILE";
}
