/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tracing;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import it.eng.spago.base.SourceBean;
import it.eng.spago.init.InitializerIFace;

public class Log4JLogger implements IFaceLogger, InitializerIFace {
    private String _name = null;
    private SourceBean _config = null;
    private Logger _logger = null;
    private boolean _isEnabled = false;
    private int _minLogSeverity = 0;
    private boolean _debug = false;

    public Log4JLogger() {
        super();
        _name = null;
        _config = null;
        _logger = null;
        _isEnabled = false;
        _minLogSeverity = 0;
        _debug = true;
    } // public Log4JLogger()

    public String getName() {
        return _name;
    } // public String getName()

    public void setName(String name) {
        _name = name;
    } // public void setName(String name)

    public synchronized void init(SourceBean config) {
        _config = config;
    } //public synchronized void init(SourceBean config)

    public synchronized SourceBean getConfig() {
        return _config;
    } //public synchronized SourceBean getConfig()

    /**
     * Abilita il tracciamento.
     */
    public synchronized void enable() {
        if (_isEnabled)
            return;
        //non deve essere tenuto in considerazione il valore definito nel file .properties
        //bens il file tracing.xml
        _logger = Logger.getLogger(_name);
        String minLogSeverity =
            (String) ((SourceBean) getConfig()).getAttribute(
                "TRACE_MIN_LOG_SEVERITY");
        _minLogSeverity = 0;
        try {
            _minLogSeverity = Integer.parseInt(minLogSeverity);
            if (_minLogSeverity < 0)
                _minLogSeverity = 0;
        } // try
        catch (Exception ex) {
            System.out.println(
                "Log4JLogger::enable: parametro TRACE_MIN_LOG_SEVERITY non valido");
            ex.printStackTrace();
        } // catch (Exception ex)
        _logger.setLevel(getLevel(_minLogSeverity));
        String debug =
            (String) ((SourceBean) getConfig()).getAttribute("DEBUG");
        _debug =
            ((debug != null) && (debug.equalsIgnoreCase("TRUE")))
                ? true
                : false;
        if (_debug)
            _logger.setLevel(Level.toLevel(Level.DEBUG_INT));
        _isEnabled = true;
    } // public synchronized void enable()

    /**
     * Disabilita il tracciamento
     */
    public synchronized void disable() {
        if (!_isEnabled)
            return;
        _logger = null;
        _isEnabled = false;
    } // public synchronized void disable()

    /**
     * Verifica se il tracer &egrave; abilitato a tracciare.
     * @return <code>boolean</code>
     */
    public synchronized boolean isEnabled() {
        return _isEnabled;
    } // public synchronized boolean isEnabled()

    /**
     * Questo metodo verifica se il parametro severity e ritorna:
     * <em>false</em> se il severity non  <em>DEBUG</em> e inferiore della severity minima.
     * <em>true</em>se il severity   <em>DEBUG</em> o maggiore della severity minima.
     * @param severity codice della severity
     * @return il risultato della comparazione.
     */
    public synchronized boolean testToTrace(int severity) {
        //se viene richiesto il tracciamento di un log con severity = DEBUG
        if (!_isEnabled)
            return false;
        if (severity == TracerSingleton.DEBUG)
            return _debug;
        else if (severity < _minLogSeverity)
            return false;
        else
            return true;
    } // public synchronized boolean testToTrace(int severity)

    public synchronized void trace(
        String application,
        int severity,
        String message) {
        if (!testToTrace(severity))
            return;
        _logger.log(getLevel(severity), message);
    } // public synchronized void trace(String application, int severity, String message)

    private Level getLevel(int severity) {
        int levelInt = 0;
        switch (severity) {
            case TracerSingleton.INFORMATION :
                levelInt = Level.INFO_INT;
                break;
            case TracerSingleton.WARNING :
                levelInt = Level.WARN_INT;
                break;
            case TracerSingleton.MINOR :
                levelInt = Level.ERROR_INT;
                break;
            case TracerSingleton.MAJOR :
                levelInt = Level.ERROR_INT;
                break;
            case TracerSingleton.CRITICAL :
                levelInt = Level.FATAL_INT;
                break;
            case TracerSingleton.DEBUG :
                levelInt = Level.DEBUG_INT;
                break;
            default :
                levelInt = Level.DEBUG_INT;
                break;
        } // switch (severity)
        return Level.toLevel(levelInt);
    } // private Level getLevel(int severity)
} // public class Log4JLogger implements IFaceLogger
