/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.impl;

import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.validator.GenericValidator;
import org.apache.commons.validator.UrlValidator;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           COntrollo su campo alfa numerico
* 										    
**/

public class SyntaxChecker 
{
	public static final String FIELD_GENERIC = "GENERIC";
	public static final String FIELD_FISCALCODE = "FISCALCODE";
	public static final String FIELD_EMAIL = "EMAIL";
	public static final String FIELD_URL = "URL";
	public static final String FIELD_DATE = "DATE";
	public static final String FIELD_NUMERIC = "NUMERIC";	
	public static final String FIELD_ALFANUMERIC = "ALFANUMERIC";
	public static final String FIELD_LETTERSTRING = "LETTERSTRING";
	public static final String FIELD_REGULAR_EXPRESSION = "REGEXP";
	public static final String FIELD_IMPORTO = "DECIMAL";
	public static final String FIELD_NAME = "NAME";
	
	public static final String FIELD_GENERIC_DEPRECATED = "1";
	public static final String FIELD_FISCALCODE_DEPRECATED = "2";
	public static final String FIELD_EMAIL_DEPRECATED = "3";
	public static final String FIELD_URL_DEPRECATED = "4";
	public static final String FIELD_DATE_DEPRECATED = "5";
	public static final String FIELD_NUMERIC_DEPRECATED = "6";	
	public static final String FIELD_ALFANUMERIC_DEPRECATED = "7";
	public static final String FIELD_LETTERSTRING_DEPRECATED = "8";
	public static final String FIELD_REGULAR_EXPRESSION_DEPRECATED = "10";	
	public static final String FIELD_IMPORTO_DEPRECATED = "11";
	public static final String FIELD_NAME_DEPRECATED = "12";
	
	// Espressioni regolari per la gestione dell'attributo "regExp"
	private static final String DIGIT_REGEXP = "^([0-9])*$";
	private static final String CODICE_FISCALE_REGEXP =
		"^([A-Z]{6}[A-Z\\d]{2}[A-Z][A-Z\\d]{2}[A-Z][A-Z\\d]{3}[A-Z])*$";

	private static final String LETTER_STRING_REGEXP = "^([a-zA-Z])*$";
	private static final String ALPHANUMERIC_STRING_REGEXP="^([a-zA-Z0-9\\s])*$";
	private static final String PERSON_NAME_REGEXP = "^([a-zA-Z' .,])*$";
	
	// Codici Errori
	public static final int ERRORE_CAMPO_OBBLIGATORIO = 10100;
	public static final int ERRORE_CAMPO_CODICE_FISCALE = 10101;
	public static final int ERRORE_CAMPO_PARTITA_IVA = 10102;
	public static final int ERRORE_CAMPO_EMAIL = 10103; 		
	public static final int ERRORE_CAMPO_SITO = 10104; 		
	public static final int ERRORE_CAMPO_DATA = 10105; 		
	public static final int ERRORE_CAMPO_NUMERICO = 10106; 		
	public static final int ERRORE_CAMPO_ALFANUMERICO = 10107; 		
	public static final int ERRORE_CAMPO_LETTERALE = 10108;
	public static final int ERRORE_CAMPO_RE = 10110; 		
	public static final int ERRORE_CAMPO_IMPORTO = 10111; 		
	public static final int ERRORE_CODICE_CONTROLLO = 10112;
	public static final int ERRORE_CAMPO_NOMINATIVO= 10113;
	public static final int ERRORE_CIFRE_DECIMALI= 10114;
	public static final int ERRORE_CODICE_DIPENDENZE = 10115;
	public static final int ERRORE_CODICE_ALL_OR_NO = 10116;
	public static final int ERRORE_MAXLENGTH = 10117;		
	
	public static final String DEFAULT_RANGE = "";
	public static final int DEFAULT_CIFREDECIMALI = -1;
	public static final int DEFAULT_MAXCIFREDECIMALI = -1;

	/**
	 * Method check. Verifica la validit di un campo di input.
	 * @param mandatory true se  il campo  obbligatorio, false altrimenti.
	 * @param tipoCampo tipo del campo, deve essere uno dei valori CAMPO_CODICE_FISCALE..CAMPO_DATA
	 * @param value valore da verificare
	 * @return boolean true se il campo supera i controlli formali, false altrimenti
	 */
	public static boolean check(
		final boolean mandatory,
		final int maxlength,
		final String fieldType,
		final String value,
		final String fieldName,
		final String regExp,
		final int decimals,
		final int maxDecimals,

		EMFErrorHandler errorHandler) {
		boolean risultato = true;
		int codice_errore=ERRORE_CODICE_CONTROLLO;
		List params = new ArrayList();
		params.add(fieldName);
		
		//Controllo campo obbligatorio
		if (mandatory && GenericValidator.isBlankOrNull(value))
		{
			codice_errore = ERRORE_CAMPO_OBBLIGATORIO;
			risultato = false;
		}

		//Controllo maxlength
		if (risultato && maxlength!=-1  
			&& (value!=null && value.length()>maxlength) )
		{
			codice_errore = ERRORE_MAXLENGTH;
			risultato = false;
		}

		if (risultato && !GenericValidator.isBlankOrNull(value))	
		{
			if ((fieldType == null) || fieldType.equalsIgnoreCase(FIELD_GENERIC) || fieldType.equalsIgnoreCase(FIELD_GENERIC_DEPRECATED)) {
				// No checks
		    } else if (fieldType.equalsIgnoreCase(FIELD_FISCALCODE) || fieldType.equalsIgnoreCase(FIELD_FISCALCODE_DEPRECATED)) {

				char primoChar = value.charAt(0);
				// Verifico se  un codice fiscale o una partita iva
				if(GenericValidator.matchRegexp(String.valueOf(primoChar), DIGIT_REGEXP))
				{
					if(gestionePI(value,11)==null)
						risultato = false;
					codice_errore = ERRORE_CAMPO_PARTITA_IVA;
				}		
				else
				{
					risultato = controllaCF(value);
					codice_errore = ERRORE_CAMPO_CODICE_FISCALE;
				}	
		    	
		    } else if (fieldType.equalsIgnoreCase(FIELD_EMAIL) || fieldType.equalsIgnoreCase(FIELD_EMAIL_DEPRECATED)) {
				risultato = GenericValidator.isEmail(value);
				codice_errore = ERRORE_CAMPO_EMAIL;
		    } else if (fieldType.equalsIgnoreCase(FIELD_URL) || fieldType.equalsIgnoreCase(FIELD_URL_DEPRECATED)) {
		        UrlValidator urlValidator = new UrlValidator();
				risultato = urlValidator.isValid(value);
				codice_errore = ERRORE_CAMPO_SITO;
		    } else if (fieldType.equalsIgnoreCase(FIELD_DATE) || fieldType.equalsIgnoreCase(FIELD_DATE_DEPRECATED)) {

		    	// Actually the date format is fixed: should be configurable
				if(StringUtils.stringToDate(value) == null)
					risultato = false;
				else
				{
					// Verifico che l'anno rispetti il formato yyyy
					String anno = value.substring(value.lastIndexOf("/")+1);
					if(anno.length()<4)
						risultato = false;
				}	
				codice_errore = ERRORE_CAMPO_DATA;
		    	
		    } else if (fieldType.equalsIgnoreCase(FIELD_NUMERIC) || fieldType.equalsIgnoreCase(FIELD_NUMERIC_DEPRECATED)) {
				risultato = isNumber(value);
				codice_errore = ERRORE_CAMPO_NUMERICO;
		    } else if (fieldType.equalsIgnoreCase(FIELD_ALFANUMERIC) || fieldType.equalsIgnoreCase(FIELD_ALFANUMERIC_DEPRECATED)) {
				risultato = GenericValidator.matchRegexp(value, ALPHANUMERIC_STRING_REGEXP);
				codice_errore = ERRORE_CAMPO_ALFANUMERICO;
		    } else if (fieldType.equalsIgnoreCase(FIELD_LETTERSTRING) || fieldType.equalsIgnoreCase(FIELD_LETTERSTRING_DEPRECATED)) {
				risultato = GenericValidator.matchRegexp(value, LETTER_STRING_REGEXP);
				codice_errore = ERRORE_CAMPO_LETTERALE;
		    } else if (fieldType.equalsIgnoreCase(FIELD_REGULAR_EXPRESSION) || fieldType.equalsIgnoreCase(FIELD_REGULAR_EXPRESSION_DEPRECATED)) {
			    risultato = GenericValidator.matchRegexp(value, regExp);
				codice_errore = ERRORE_CAMPO_RE;
				params.add(regExp);
		    } else if (fieldType.equalsIgnoreCase(FIELD_IMPORTO) || fieldType.equalsIgnoreCase(FIELD_IMPORTO_DEPRECATED)) {
		        
		    	// Actually the number format is fixed: should be configurable
				// Retrieve the decimal part of the string: for example in "123,45" is "45"
				int pos = value.indexOf(",");
				String cifre = "";
				if(pos!=-1)
					cifre = value.substring(pos+1);

				// Check if the number of decimals exceeds the declared maximum number
				if((decimals==-1) && (maxDecimals!=-1) && (cifre.length()>maxDecimals))
				{
					risultato = false;
					params.add(String.valueOf(maxDecimals));							
					codice_errore = ERRORE_CAMPO_IMPORTO;
				}	
				
				// Check if the of decimals is exactly as declared
				if ((decimals!=-1) && (cifre.length()!=decimals))
				{
					risultato = false;
					params.add(String.valueOf(decimals));							
					codice_errore = ERRORE_CAMPO_IMPORTO;
				}	
				
				if(risultato && (regExp!=null) && !regExp.equals(""))
				{
				    risultato = GenericValidator.matchRegexp(value, regExp);
					params.add(regExp);
					codice_errore = ERRORE_CIFRE_DECIMALI;
				}
		    	
		    } else if (fieldType.equalsIgnoreCase(FIELD_NAME) || fieldType.equalsIgnoreCase(FIELD_NAME_DEPRECATED)) {
				risultato = GenericValidator.matchRegexp(value, PERSON_NAME_REGEXP);
				params.add("A-Za-z',. ");
				codice_errore = ERRORE_CAMPO_NOMINATIVO;
		    } else {
				risultato=false;
				codice_errore = ERRORE_CODICE_CONTROLLO;
				params.clear();
				params.add(String.valueOf(fieldType));
		    }
	        
		}
		
		if(!risultato)
			errorHandler.addError(new EMFUserError(EMFErrorSeverity.ERROR, codice_errore,params));	        	

		return risultato;
	}	

	private static boolean isNumber(String value)
	{
	    // Queste funzioni considerano come separatore dei decimali il .
	    return (GenericValidator.isInt(value) 
				|| GenericValidator.isFloat(value) 
				|| GenericValidator.isDouble(value)
				|| GenericValidator.isShort(value)
				|| GenericValidator.isLong(value));
		//return GenericValidator.matchRegexp(value, DIGIT_REGEXP);
	}	
	
	/**
	 * Verifica il formato della Partita IVA.
	 * Ritorna la Partita IVA completata di zeri
	 * eventualmente.
	 */
	private static String gestionePI(String pi, int lng) {
		int valueLng = pi.length();
		if (valueLng <= lng) {
			if(!GenericValidator.matchRegexp(pi, DIGIT_REGEXP))
				return null;
			pi = riempiZero(pi, lng);
			if (carattereControlloCorretto(pi)) return pi;
			return null;
		} else
			return null;
	} //end gestionePI


	private static boolean carattereControlloCorretto(String pi)
	{
		int i, c, s;
		s = 0;
		for (i = 0; i <= 9; i += 2)
		{
			s += pi.charAt(i) - '0';
		}
		for (i = 1; i <= 9; i += 2) 
		{
			c = 2 * (pi.charAt(i) - '0');
			if (c > 9)
				c = c - 9;
			s += c;
		}
		if ((10 - s % 10) % 10 != pi.charAt(10) - '0')
		{
			return false;
		}
		return true;
	}
	/**
	 * Aggiunge a sinistra della stringa di input un numero 
	 * di zeri pari alla lunghezza specificata meno la 
	 * lunghezza della stringa. 
	 */
	public static String riempiZero(String value, int lng) {
		int lung = lng - value.length();
		for (int i = 0; i < lung; i++)
			 value = "0" + value;
		return value;	 
	} //end riempiZero

	/**
	 * Verifica il formato del Codice Fiscale
	 */
	private static boolean controllaCF(String cf) {
		cf = cf.toUpperCase().trim();
		return (GenericValidator.matchRegexp(cf, CODICE_FISCALE_REGEXP));
	} //end controllaCF


}
