/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.io.Serializable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class RemoteSessionFacadeEJBProxy implements Serializable {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
	 * Nome JNDI dell'EJB
	 */
	private String _jndi = null;

	/**
	 * indirizzo IP del server
	 */
	private String _adress = null;

	/**
	 * @param jndi
	 */
	public RemoteSessionFacadeEJBProxy() {
		ConfigSingleton configure = ConfigSingleton.getInstance();
		SourceBean proxy = (SourceBean) configure
				.getFilteredSourceBeanAttribute("PROXIES.PROXY", "NAME",
						"REMOTE_SESSION_FACADE_EJB_PROXY");
		if (proxy != null) {
			_jndi = (String) proxy.getAttribute("jndi");
			_adress = (String) proxy.getAttribute("adress");
		} else {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"ERRORE NELLA LETTURA DEL FILE PROXY.XML");
		}
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"RemoteSessionFacadeEJBProxy::RemoteSessionFacadeEJBProxy: _jndi ["
						+ _jndi + "] _adress [ " + _adress + "]");
	}

	/**
	 * Metodo che eseue il lookup all'EJB remoto e delega l'esecuzione del
	 * servizio. Prende l'oggetto risultato e aggiorna i Container di
	 * Request,Response e Session
	 * 
	 * @param request
	 * @return @throws
	 *         Exception
	 */
	public RequestContextIFace service(RequestContextIFace request)
			throws Exception {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"RemoteSessionFacadeEJBProxy::service: invocato");
		SessionFacadeEJBLocal sessionFacadeEJBLocal = null;
		RequestContextIFace risultato = null;
		try {
			//Hashtable env = new Hashtable();
			//env.put(Context.INITIAL_CONTEXT_FACTORY, _factory);
			//env.put(Context.PROVIDER_URL, _adress);
			//Context ctx = new InitialContext(env);
            Context ctx = new InitialContext();
			RemoteSessionFacadeEJBHome eJBHome = (RemoteSessionFacadeEJBHome) PortableRemoteObject
					.narrow(ctx.lookup(_jndi), RemoteSessionFacadeEJBHome.class);
			RemoteSessionFacadeEJB ejb = eJBHome.create();

			//            byte[]
			// datSer=Serializer.serialize(request.getRequestContainer());
			ServletConfig _servletConfig = (ServletConfig) request
					.getRequestContainer().getAdapterConfig();
			HttpServletResponse _httpResponse = (HttpServletResponse) request
					.getRequestContainer().getInternalResponse();
			HttpServletRequest _httpRequest = (HttpServletRequest) request
					.getRequestContainer().getInternalRequest();
			// sono obbligato ad annullare questi 3 oggetti perch non sono
			// serializzabili
			request.getRequestContainer().setAdapterConfig(null);
			request.getRequestContainer().setInternalResponse(null);
			request.getRequestContainer().setInternalRequest(null);
			risultato = ejb.service(request);
			request.getRequestContainer().setAdapterConfig(_servletConfig);
			request.getRequestContainer().setInternalResponse(_httpResponse);
			request.getRequestContainer().setInternalRequest(_httpRequest);
			// aggiorno le variabili locali con la sessione e il risultato
			request.getResponseContainer().setServiceResponse(
					risultato.getResponseContainer().getServiceResponse());
			request.getRequestContainer().setServiceRequest(
					risultato.getRequestContainer().getServiceRequest());
			request.getRequestContainer().setSessionContainer(
					risultato.getRequestContainer().getSessionContainer());
		}

		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"SessionFacadeEJBLocalProxy::service:", ex);
		} finally {
			if (sessionFacadeEJBLocal != null)
				sessionFacadeEJBLocal.remove();
		}
		return risultato;
	}
}
