/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.base;

import it.eng.spago.tracing.TracerSingleton;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * La classe <code>RequestContainerAccess</code> permette di accede all'oggetto
 * <code>RequestContainer</code> <p>
 * @version 1.0, 15/01/2003
 * @author Daniela Butano
 * @see BaseContainer
 * @see SessionContainer
 * @see ApplicationContainer
 * @see ResponseContainer
 */
public class RequestContainerAccess {
    /**
     * Ritorna il <code>RequestContainer</code> il cui riferimento &egrave;
     * memorizzato nell'oggetto <code>HttpServletRequest</code> <em>request</em>
     * della richiesta HTTP corrente.
     * <p>
     * @param request <code>HttpServletRequest</code> della richiesta HTTP corrente
     * @return <code>RequestContainer</code> il contenitore dei dati di sessione
     */
    public static RequestContainer getRequestContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerAccess::getRequestContainer: request nullo");
            return null;
        } // if (request == null)
        String requestContainerName = request.getParameter(Constants.REQUEST_CONTAINER);
        if (requestContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerAccess::getRequestContainer: requestContainerName nullo");
            return null;
        } // if (requestContainerName == null)
        RequestContainer requestContainer = (RequestContainer)request.getAttribute(requestContainerName);
        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerAccess::getRequestContainer: requestContainer non contenuto in request");
            HttpSession session = request.getSession(true);
            requestContainer = (RequestContainer)session.getAttribute(requestContainerName);
            if (requestContainer == null)
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "RequestContainerAccess::getRequestContainer: requestContainer non contenuto in sessione");
            else
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "RequestContainerAccess::getRequestContainer: requestContainer contenuto in sessione");
        } // if (requestContainer == null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "RequestContainerAccess::getRequestContainer: requestContainer contenuto in request");
        return requestContainer;
    } // public static RequestContainer getRequestContainer(HttpServletRequest request)

    public static void delRequestContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerAccess::delRequestContainer: request nullo");
            return;
        } // if (request == null)
        String requestContainerName = request.getParameter(Constants.REQUEST_CONTAINER);
        if (requestContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerAccess::delRequestContainer: requestContainerName nullo");
            return;
        } // if (requestContainerName == null)
        request.setAttribute(requestContainerName, null);
        HttpSession session = request.getSession(true);
        session.setAttribute(requestContainerName, null);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "RequestContainerAccess::delRequestContainer: requestContainer rimosso");
    } // public static void delRequestContainer(HttpServletRequest request)
} // public class RequestContainerAccess
