/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.base;

import it.eng.spago.tracing.TracerSingleton;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * La classe <code>ResponseContainerAccess</code> permette di accede all'oggetto
 * <code>ResponseContainer</code> <p>
 * @version 1.0, 15/01/2003
 * @author Daniela Butano
 * @see BaseContainer
 * @see SessionContainer
 * @see ApplicationContainer
 * @see ResponseContainer
 */
public class ResponseContainerAccess {
    /**
     * Ritorna il <code>ResponseContainer</code> il cui riferimento &egrave;
     * memorizzato nell'oggetto <code>HttpServletRequest</code> <em>request</em>
     * della richiesta HTTP corrente.
     * <p>
     * @param request <code>HttpServletRequest</code> della richiesta HTTP corrente
     * @return <code>ResponseContainer</code> il contenitore dei dati di sessione
     */
    public static ResponseContainer getResponseContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerAccess::getResponseContainer: request nullo");
            return null;
        } // if (request == null)
        String responseContainerName = request.getParameter(Constants.RESPONSE_CONTAINER);
        if (responseContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerAccess::getResponseContainer: responseContainerName nullo");
            return null;
        } // if (responseContainerName == null)
        ResponseContainer responseContainer = (ResponseContainer)request.getAttribute(responseContainerName);
        if (responseContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerAccess::getResponseContainer: responseContainer non contenuto in request");
            HttpSession session = request.getSession(true);
            responseContainer = (ResponseContainer)session.getAttribute(responseContainerName);
            if (responseContainer == null)
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ResponseContainerAccess::getResponseContainer: responseContainer non contenuto in sessione");
            else
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "ResponseContainerAccess::getResponseContainer: responseContainer contenuto in sessione");
        } // if (requestContainer == null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ResponseContainerAccess::getResponseContainer: responseContainer contenuto in request");
        return responseContainer;
    } // public static ResponseContainer getResponseContainer(HttpServletRequest request)

    public static void delResponseContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerAccess::delResponseContainer: request nullo");
            return;
        } // if (request == null)
        String responseContainerName = request.getParameter(Constants.RESPONSE_CONTAINER);
        if (responseContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerAccess::delResponseContainer: responseContainerName nullo");
            return;
        } // if (responseContainerName == null)
        request.setAttribute(responseContainerName, null);
        HttpSession session = request.getSession(true);
        session.setAttribute(responseContainerName, null);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ResponseContainerAccess::delResponseContainer: responseContainer rimosso");
    } // public static void delResponseContainer(HttpServletRequest request)
} // public class ResponseContainerAccess
