/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.service;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class DefaultHttpRequestContext extends DefaultRequestContext implements HttpRequestContextIFace {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private HttpServletRequest _httpRequest = null;
    private HttpServletResponse _httpResponse = null;
    private ServletConfig _servletConfig = null;
    private boolean _httpResponseFreezed = false;

    public DefaultHttpRequestContext() {
        super();
        _httpRequest = null;
        _httpResponse = null;
        _servletConfig = null;
        _httpResponseFreezed = false;
    } // public DefaultHttpRequestContext()

    public DefaultHttpRequestContext(RequestContainer requestContainer, ResponseContainer responseContainer) {
        super(requestContainer, responseContainer);
        _httpRequest = (HttpServletRequest)requestContainer.getInternalRequest();
        _httpResponse = (HttpServletResponse)requestContainer.getInternalResponse();
        _servletConfig = (ServletConfig)requestContainer.getAdapterConfig();
        _httpResponseFreezed = false;
    } // public DefaultHttpRequestContext(RequestContainer requestContainer,

    // ResponseContainer responseContainer)
    public void setRequestContext(RequestContextIFace requestContext) {
        super.setRequestContext(requestContext);
        if (requestContext == null) {
            _httpRequest = null;
            _httpResponse = null;
            _servletConfig = null;
        } // if (requestContext == null)
        else {
            _httpRequest = (HttpServletRequest)requestContext.getRequestContainer().getInternalRequest();
            _httpResponse = (HttpServletResponse)requestContext.getRequestContainer().getInternalResponse();
            _servletConfig = (ServletConfig)requestContext.getRequestContainer().getAdapterConfig();
        } // if (requestContext == null) else
    } // public void setRequestContext(RequestContextIFace requestContext)

    public HttpServletRequest getHttpRequest() {
        return _httpRequest;
    } // public HttpServletRequest getHttpRequest()

    public HttpServletResponse getHttpResponse() {
        return _httpResponse;
    } // public HttpServletResponse getHttpResponse()

    public ServletConfig getServletConfig() {
        return _servletConfig;
    } // public ServletConfig getServletConfig()

    public void freezeHttpResponse() {
        _httpResponseFreezed = true;
        getResponseContainer().setAttribute(Constants.HTTP_RESPONSE_FREEZED, new Boolean(_httpResponseFreezed));
    } // public void freezeHttpResponse

    public void unfreezeHttpResponse() {
        _httpResponseFreezed = false;
        getResponseContainer().setAttribute(Constants.HTTP_RESPONSE_FREEZED, new Boolean(_httpResponseFreezed));
    } // public void unfreezeHttpResponse

    public boolean isHttpResponseFreezed() {
        return _httpResponseFreezed;
    } // public boolean isHttpResponseFreezed()
} // public class DefaultHttpRequestContext extends DefaultRequestContext implements HttpRequestContextIFace
