/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tags;

import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.navigation.LabelID;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.tracing.TracerSingleton;

public class DefaultNavigationToolbarTag extends TagSupport {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    public static final String NAVIGATOR_SERVICE = "NAVIGATOR_SERVICE";

    public int doStartTag() throws JspException {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultNavigationToolbarTag::doStartTag:: invocato");
        if (pageContext == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } // if (_httpRequest == null)
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag::",
                ex);
            throw new JspException(ex.getMessage());
        } // catch (Exception ex)
        if (httpRequest == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        } // if (_httpRequest == null)
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if (responseContainer == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag:: responseContainer nullo");
            throw new JspException("responseContainer nullo");
        } // if (responseContainer == null)
        StringBuffer output = new StringBuffer();

        RequestContainer requestContainer =
            RequestContainerAccess.getRequestContainer(httpRequest);

        generateToolbar(output, requestContainer, responseContainer);
        try {
            pageContext.getOut().print(output.toString());
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultNavigationToolbarTag::doStartTag::",
                ex);
            throw new JspException(ex.getMessage());
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    public int doEndTag() throws JspException {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DefaultNavigationToolbarTag::doEndTag:: invocato");
        return super.doEndTag();
    } // public int doEndTag() throws JspException

    protected void generateToolbar(
        StringBuffer output,
        RequestContainer requestContainer,
        ResponseContainer responseContainer)
        throws JspException {
        output.append("<TABLE><tr>");
        Vector ordered = new Vector();
        RequestContainer container = requestContainer;
        if (container != null)
            container = (RequestContainer) container.getParent();
        while (container != null) {
            LabelID labelID =
                (LabelID) container.getAttribute(NAVIGATOR_SERVICE);
            if (labelID != null) {
                ordered.addElement(labelID);
            } //if (label != null)
            container = (RequestContainer) container.getParent();
        } // while (container != null)
        //scorro vettore in ordine inverso (da pi vecchia a pi recente) e creo html
        for (int i = ordered.size() - 1; i >= 0; i--) {
            LabelID label = (LabelID) ordered.elementAt(i);
            output.append(
                "<td><a href=\"AdapterHTTP?"
                    + Navigator.NAVIGATOR_BACK_TO_SERVICE_ID
                    + "=");
            output.append(label.getId());
            output.append("\" name =\"pagina\">");
            output.append(label.getLabel());
            output.append("</a> > </td>");
        } //for(int i = ordered.size()-1; i >= 0; i--)
        output.append("</tr></TABLE>");
    } // protected void generateToolbar(StringBuffer output, SourceBean _layout, RequestContainer requestContainer,
} //public class DefaultNavigationToolbarTag extends TagSupport
